"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var events_1 = require("../../events");
var __1 = require("..");
var provider_1 = require("../provider");
var cancel_manager_1 = require("../cancel-manager");
var logger_1 = require("../../utils/logger");
var TaskGraphLoader = (function () {
    function TaskGraphLoader(taskEventBinder, runnableTaskDescriptor, taskVerifier, taskProvider, cancelManager) {
        if (taskEventBinder === void 0) { taskEventBinder = events_1.on; }
        if (runnableTaskDescriptor === void 0) { runnableTaskDescriptor = __1.run; }
        if (taskVerifier === void 0) { taskVerifier = provider_1.checkIfTaskExists; }
        if (taskProvider === void 0) { taskProvider = provider_1.getTask; }
        if (cancelManager === void 0) { cancelManager = cancel_manager_1.taskCancelManager; }
        this.taskEventBinder = taskEventBinder;
        this.runnableTaskDescriptor = runnableTaskDescriptor;
        this.taskVerifier = taskVerifier;
        this.taskProvider = taskProvider;
        this.cancelManager = cancelManager;
        this.graphTasks = new Set();
    }
    TaskGraphLoader.prototype.load = function (graph) {
        return __awaiter(this, void 0, void 0, function () {
            var createEventListener, planTaskToBeRun;
            var _this = this;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (this.loadingTaskGraph) {
                            throw new Error("Loading more than one task graph is not permitted");
                        }
                        createEventListener = function (eventName, taskBuilder) { return _this.bindTaskToStartAndCancelEvent(eventName, taskBuilder); };
                        planTaskToBeRun = function (taskName) {
                            return _this.trackTaskGoingToBeRun(taskName);
                        };
                        this.getLogger().info("Loading task graph");
                        this.loadingTaskGraph = graph.describe(createEventListener, planTaskToBeRun);
                        return [4, this.loadingTaskGraph];
                    case 1:
                        _a.sent();
                        return [4, this.cancelManager.init()];
                    case 2:
                        _a.sent();
                        return [2];
                }
            });
        });
    };
    TaskGraphLoader.prototype.isReady = function () {
        return __awaiter(this, void 0, void 0, function () {
            var tasksToBePrepared;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4, this.tasksNotReady()];
                    case 1:
                        tasksToBePrepared = _a.sent();
                        return [2, tasksToBePrepared.length === 0];
                }
            });
        });
    };
    TaskGraphLoader.prototype.prepare = function () {
        return __awaiter(this, void 0, void 0, function () {
            var tasksToBePrepared;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4, this.tasksNotReady()];
                    case 1:
                        tasksToBePrepared = _a.sent();
                        this.getLogger().info(tasksToBePrepared.length + " are up to be prepared");
                        return [4, Promise.all(tasksToBePrepared.map(function (task) { return task.prepare(); }))];
                    case 2:
                        _a.sent();
                        return [2];
                }
            });
        });
    };
    TaskGraphLoader.prototype.tasksNotReady = function () {
        return __awaiter(this, void 0, void 0, function () {
            var e_1, _a, tasksToBePrepared, _b, _c, task, hasToBePrepared, e_1_1;
            return __generator(this, function (_d) {
                switch (_d.label) {
                    case 0:
                        if (!this.loadingTaskGraph) {
                            throw new Error("Load a task graph first!");
                        }
                        return [4, this.loadingTaskGraph];
                    case 1:
                        _d.sent();
                        tasksToBePrepared = [];
                        _d.label = 2;
                    case 2:
                        _d.trys.push([2, 7, 8, 9]);
                        _b = __values(this.graphTasks), _c = _b.next();
                        _d.label = 3;
                    case 3:
                        if (!!_c.done) return [3, 6];
                        task = _c.value;
                        return [4, this.hasToBePrepared(task)];
                    case 4:
                        hasToBePrepared = _d.sent();
                        if (hasToBePrepared) {
                            tasksToBePrepared.push(task);
                        }
                        _d.label = 5;
                    case 5:
                        _c = _b.next();
                        return [3, 3];
                    case 6: return [3, 9];
                    case 7:
                        e_1_1 = _d.sent();
                        e_1 = { error: e_1_1 };
                        return [3, 9];
                    case 8:
                        try {
                            if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
                        }
                        finally { if (e_1) throw e_1.error; }
                        return [7];
                    case 9: return [2, tasksToBePrepared];
                }
            });
        });
    };
    TaskGraphLoader.prototype.bindTaskToStartAndCancelEvent = function (eventName, taskBuilder) {
        this.taskEventBinder(eventName, taskBuilder);
    };
    TaskGraphLoader.prototype.trackTaskGoingToBeRun = function (taskName) {
        this.taskVerifier(taskName);
        this.graphTasks.add(this.taskProvider(taskName));
        return this.runnableTaskDescriptor(taskName);
    };
    TaskGraphLoader.prototype.hasToBePrepared = function (task) {
        return __awaiter(this, void 0, void 0, function () {
            var err_1;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        _a.trys.push([0, 2, , 3]);
                        return [4, task.checkIfCanRun()];
                    case 1:
                        _a.sent();
                        return [2, false];
                    case 2:
                        err_1 = _a.sent();
                        return [2, true];
                    case 3: return [2];
                }
            });
        });
    };
    TaskGraphLoader.prototype.getLogger = function () {
        if (!this.logger) {
            this.logger = logger_1.getLogger("TaskGraphLoader");
        }
        return this.logger;
    };
    return TaskGraphLoader;
}());
exports.TaskGraphLoader = TaskGraphLoader;
exports.taskGraph = new TaskGraphLoader();
//# sourceMappingURL=loader.js.map