"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var provider_1 = require("./provider");
var TaskManager = (function () {
    function TaskManager(planningType, plannedTasksStore, intervalOffset, currentTime) {
        if (currentTime === void 0) { currentTime = new Date().getTime(); }
        this.planningType = planningType;
        this.plannedTasksStore = plannedTasksStore;
        this.intervalOffset = intervalOffset;
        this.currentTime = currentTime;
    }
    TaskManager.prototype.tasksToRun = function () {
        return __awaiter(this, void 0, void 0, function () {
            var allTasks, tasksToRunNow;
            var _this = this;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4, this.allTasks()];
                    case 1:
                        allTasks = _a.sent();
                        tasksToRunNow = allTasks.filter(function (task) {
                            return _this.determineIfTaskShouldBeRun(task);
                        });
                        return [2, tasksToRunNow];
                }
            });
        });
    };
    TaskManager.prototype.requiresForeground = function () {
        return __awaiter(this, void 0, void 0, function () {
            var tasksToRun, allRunInBackground;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4, this.tasksToRun()];
                    case 1:
                        tasksToRun = _a.sent();
                        allRunInBackground = tasksToRun.every(function (plannedTask) {
                            return provider_1.getTask(plannedTask.name).runsInBackground();
                        });
                        return [2, !allRunInBackground];
                }
            });
        });
    };
    TaskManager.prototype.willContinue = function () {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4, this.nextInterval()];
                    case 1: return [2, (_a.sent()) !== -1];
                }
            });
        });
    };
    TaskManager.prototype.nextInterval = function () {
        return __awaiter(this, void 0, void 0, function () {
            var allTasks, tasksToRunInTheFuture, nextIntervals, sortedIntervals;
            var _this = this;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4, this.allTasks()];
                    case 1:
                        allTasks = _a.sent();
                        if (allTasks.length === 0) {
                            return [2, -1];
                        }
                        tasksToRunInTheFuture = allTasks.filter(function (task) { return task.recurrent || !_this.determineIfTaskShouldBeRun(task); });
                        if (tasksToRunInTheFuture.length === 0) {
                            return [2, -1];
                        }
                        nextIntervals = tasksToRunInTheFuture.map(function (task) {
                            var next = task.nextRun(_this.currentTime);
                            if (next < _this.intervalOffset) {
                                next += task.interval;
                            }
                            return next;
                        });
                        sortedIntervals = nextIntervals.sort(function (i1, i2) { return i1 - i2; });
                        return [2, sortedIntervals[0]];
                }
            });
        });
    };
    TaskManager.prototype.determineIfTaskShouldBeRun = function (task) {
        if (this.currentTime < task.startAt - this.intervalOffset) {
            return false;
        }
        else if (task.lastUpdate < task.startAt) {
            return true;
        }
        return (this.currentTime >= task.lastUpdate + task.interval - this.intervalOffset);
    };
    TaskManager.prototype.allTasks = function () {
        return __awaiter(this, void 0, void 0, function () {
            var _a;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        if (!!this._allTasks) return [3, 2];
                        _a = this;
                        return [4, this.plannedTasksStore.getAllSortedByNextRun(this.planningType)];
                    case 1:
                        _a._allTasks = _b.sent();
                        _b.label = 2;
                    case 2: return [2, this._allTasks];
                }
            });
        });
    };
    return TaskManager;
}());
exports.TaskManager = TaskManager;
//# sourceMappingURL=manager.js.map