"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var events_1 = require("../../events");
var time_based_1 = require("../schedulers/time-based");
var planned_tasks_store_1 = require("../../persistence/planned-tasks-store");
var provider_1 = require("../provider");
var instant_task_runner_1 = require("../schedulers/immediate/instant-task-runner");
var task_1 = require("../task");
var cancel_manager_1 = require("../cancel-manager");
var TaskPlanner = (function () {
    function TaskPlanner(taskScheduler, taskRunner, taskStore, cancelManager) {
        if (taskRunner === void 0) { taskRunner = new instant_task_runner_1.InstantTaskRunner(planned_tasks_store_1.plannedTasksDB); }
        if (taskStore === void 0) { taskStore = planned_tasks_store_1.plannedTasksDB; }
        if (cancelManager === void 0) { cancelManager = cancel_manager_1.taskCancelManager; }
        this.taskScheduler = taskScheduler;
        this.taskRunner = taskRunner;
        this.taskStore = taskStore;
        this.cancelManager = cancelManager;
    }
    TaskPlanner.prototype.plan = function (runnableTask, dispatchableEvent) {
        return __awaiter(this, void 0, void 0, function () {
            var plannedTask, err_1;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        _a.trys.push([0, 2, , 3]);
                        provider_1.checkIfTaskExists(runnableTask.name);
                        return [4, (runnableTask.interval > 0 ||
                                runnableTask.startAt !== -1
                                ? this.planScheduled(runnableTask, dispatchableEvent)
                                : this.planImmediate(runnableTask, dispatchableEvent))];
                    case 1:
                        plannedTask = _a.sent();
                        return [2, plannedTask];
                    case 2:
                        err_1 = _a.sent();
                        this.emitTaskChainFinished(dispatchableEvent, err_1);
                        throw err_1;
                    case 3: return [2];
                }
            });
        });
    };
    TaskPlanner.prototype.planImmediate = function (runnableTask, dispatchableEvent) {
        return __awaiter(this, void 0, void 0, function () {
            var existedBefore, plannedTask;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4, this.taskStore.get(runnableTask)];
                    case 1:
                        existedBefore = _a.sent();
                        return [4, this.taskRunner.run(runnableTask, dispatchableEvent)];
                    case 2:
                        plannedTask = _a.sent();
                        if (!existedBefore) {
                            this.cancelManager.add(plannedTask);
                        }
                        return [2, plannedTask];
                }
            });
        });
    };
    TaskPlanner.prototype.planScheduled = function (runnableTask, dispatchableEvent) {
        return __awaiter(this, void 0, void 0, function () {
            var possibleExisting, plannedTask;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4, this.taskStore.get(runnableTask)];
                    case 1:
                        possibleExisting = _a.sent();
                        if (possibleExisting) {
                            this.emitTaskChainFinished(dispatchableEvent);
                            return [2, possibleExisting];
                        }
                        return [4, this.getTaskScheduler().schedule(runnableTask)];
                    case 2:
                        plannedTask = _a.sent();
                        this.emitTaskChainFinished(dispatchableEvent);
                        this.cancelManager.add(plannedTask);
                        return [2, plannedTask];
                }
            });
        });
    };
    TaskPlanner.prototype.emitTaskChainFinished = function (dispatchableEvent, error) {
        if (!dispatchableEvent) {
            return;
        }
        var result = { status: task_1.TaskResultStatus.Ok };
        if (error) {
            result = { status: task_1.TaskResultStatus.Error, reason: error };
        }
        events_1.emit(events_1.createEvent(events_1.TaskDispatcherEvent.TaskChainFinished, {
            id: dispatchableEvent.id,
            data: { result: result },
        }));
    };
    TaskPlanner.prototype.getTaskScheduler = function () {
        if (!this.taskScheduler) {
            this.taskScheduler = time_based_1.taskScheduler();
        }
        return this.taskScheduler;
    };
    return TaskPlanner;
}());
exports.TaskPlanner = TaskPlanner;
//# sourceMappingURL=index.js.map