"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var planned_tasks_store_1 = require("../../../../../../persistence/planned-tasks-store");
var manager_android_1 = require("./manager.android");
var manager_android_2 = require("../watchdog/manager.android");
var planned_task_1 = require("../../../../../planner/planned-task");
var logger_1 = require("../../../../../../utils/logger");
var MIN_ALARM_INTERVAL = 60000;
var AndroidAlarmScheduler = (function () {
    function AndroidAlarmScheduler(alarmManager, watchdogManager, plannedTaskStore) {
        if (alarmManager === void 0) { alarmManager = new manager_android_1.AndroidAlarmManager(); }
        if (watchdogManager === void 0) { watchdogManager = new manager_android_2.WatchdogManager(); }
        if (plannedTaskStore === void 0) { plannedTaskStore = planned_tasks_store_1.plannedTasksDB; }
        this.alarmManager = alarmManager;
        this.watchdogManager = watchdogManager;
        this.plannedTaskStore = plannedTaskStore;
        this.logger = logger_1.getLogger("AndroidAlarmScheduler");
    }
    AndroidAlarmScheduler.prototype.setup = function () {
        return __awaiter(this, void 0, void 0, function () {
            var plannedTasks;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (this.alarmManager.alarmUp && this.watchdogManager.alarmUp) {
                            return [2];
                        }
                        return [4, this.plannedTaskStore.getAllSortedByNextRun(planned_task_1.PlanningType.Scheduled)];
                    case 1:
                        plannedTasks = _a.sent();
                        if (plannedTasks.length > 0) {
                            if (!this.alarmManager.alarmUp) {
                                this.logger.info("Alarm was not up! Scheduling...");
                                this.alarmManager.set(this.calculateAlarmInterval(plannedTasks[0]));
                            }
                            if (!this.watchdogManager.alarmUp) {
                                this.logger.info("Watchdog was not up! Initializing...");
                                this.watchdogManager.set();
                            }
                        }
                        return [2];
                }
            });
        });
    };
    AndroidAlarmScheduler.prototype.schedule = function (runnableTask) {
        return __awaiter(this, void 0, void 0, function () {
            var possibleExisting, allTasks, now, plannedTask;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4, this.plannedTaskStore.get(runnableTask)];
                    case 1:
                        possibleExisting = _a.sent();
                        if (possibleExisting) {
                            return [2, possibleExisting];
                        }
                        return [4, this.plannedTaskStore.getAllSortedByNextRun(planned_task_1.PlanningType.Scheduled)];
                    case 2:
                        allTasks = _a.sent();
                        now = new Date().getTime();
                        plannedTask = new planned_task_1.PlannedTask(planned_task_1.PlanningType.Scheduled, planned_task_1.SchedulerType.Alarm, runnableTask);
                        if (allTasks.length === 0 ||
                            allTasks[0].nextRun(now) > plannedTask.nextRun(now)) {
                            this.alarmManager.set(this.calculateAlarmInterval(plannedTask));
                            if (!this.watchdogManager.alarmUp) {
                                this.watchdogManager.set();
                            }
                        }
                        return [4, this.plannedTaskStore.insert(plannedTask)];
                    case 3:
                        _a.sent();
                        this.logger.info("Task scheduled: " + JSON.stringify(plannedTask));
                        return [2, plannedTask];
                }
            });
        });
    };
    AndroidAlarmScheduler.prototype.cancel = function (id) {
        return __awaiter(this, void 0, void 0, function () {
            var possibleExisting, allTasks, now;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4, this.plannedTaskStore.get(id)];
                    case 1:
                        possibleExisting = _a.sent();
                        if (!possibleExisting) {
                            return [2];
                        }
                        return [4, this.plannedTaskStore.getAllSortedByNextRun(planned_task_1.PlanningType.Scheduled)];
                    case 2:
                        allTasks = _a.sent();
                        now = new Date().getTime();
                        if (allTasks.length === 1) {
                            this.alarmManager.cancel();
                            this.watchdogManager.cancel();
                        }
                        else if (allTasks[0].nextRun(now) === possibleExisting.nextRun(now) &&
                            allTasks[1].nextRun(now) !== possibleExisting.nextRun(now)) {
                            this.alarmManager.set(this.calculateAlarmInterval(allTasks[1]));
                        }
                        return [4, this.plannedTaskStore.delete(id)];
                    case 3:
                        _a.sent();
                        this.logger.info("Task with id=" + id + " has been canceled");
                        return [2];
                }
            });
        });
    };
    AndroidAlarmScheduler.prototype.calculateAlarmInterval = function (plannedTask) {
        var nextRun = plannedTask.nextRun();
        return nextRun > MIN_ALARM_INTERVAL ? nextRun : MIN_ALARM_INTERVAL;
    };
    return AndroidAlarmScheduler;
}());
exports.AndroidAlarmScheduler = AndroidAlarmScheduler;
//# sourceMappingURL=scheduler.android.js.map